/***************************************************************************
 *   Copyright (C) 2007  Martin Meredith                                   *
 *   mez@ubuntu.com                                                        *
 *                                                                         *
 *   Copyright (C) 2006  Jonathan Riddell                                  *
 *   jriddell@ubuntu.com                                                   *
 *                                                                         *
 *   Copyright (C) 2005 by Joe Ferris                                      *
 *   jferris@optimistictech.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kservice.h> 
#include <kglobal.h>
#include <kiconloader.h>
#include <kapplication.h>

#include <qclipboard.h>

#include "execcatalog.h"
#include "command.h"


Command::Command(ExecCatalog* catalog, const QString& text): KatapultItem(), _catalog(catalog), _text(text)
{
}

Command::~Command() {
}

QPixmap Command::icon(int size) const
{
	return KGlobal::iconLoader()->loadIcon("konsole", KIcon::NoGroup, size);
}

QString Command::text() const
{
	return _text;
}

void Command::setText(const QString& text)
{
	_text = text;
}

ExecCatalog* Command::catalog() const
{
	return _catalog;
}


#include "command.moc"
