/***************************************************************************
 *   Copyright (C) 2007  Martin Meredith                                   *
 *   mez@ubuntu.com                                                        *
 *                                                                         *
 *   Copyright (C) 2006  Jonathan Riddell                                  *
 *   jriddell@ubuntu.com                                                   *
 *                                                                         *
 *   Copyright (C) 2005 by Joe Ferris                                      *
 *   jferris@optimistictech.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <krun.h>

#include "execcatalog.h"
#include "command.h"
#include "katapultitem.h"
#include "actionrun.h"

ActionRun::ActionRun()
	: KatapultAction(), _command(0)
{
}

ActionRun::~ActionRun()
{
}

QString ActionRun::text() const
{
	return i18n("Execute");
}

QPixmap ActionRun::icon(int size) const
{
	return KGlobal::iconLoader()->loadIcon("exec", KIcon::NoGroup, size);
}

bool ActionRun::accepts(const KatapultItem* item) const
{
	bool accept = strcmp(item->className(), "Command") == 0;
	if (accept) {
		_command = (const Command*)item;
	}
	return accept;
}

void ActionRun::execute(const KatapultItem* item) const
{
	if (strcmp(item->className(), "Command") == 0) {
		_command = (const Command*)item;

	KRun::runCommand(_command->text().mid(_command->catalog()->triggerWordLength() + 1));


	}
}
